@@ -47,6 +47,7 @@ HunspellConfigureOptions="--with-warnings --disable-nls --disable-static"
 Qt4ConfigureOptions="${QtConfigureOptions} -opensource -silent -shared -fast -no-exceptions"
 Qt4ConfigureOptions="${Qt4ConfigureOptions} -no-webkit -no-qt3support -no-javascript-jit -no-dbus"
 Qt4ConfigureOptions="${Qt4ConfigureOptions} -nomake examples -nomake demos -nomake docs -nomake tools"
+Qt4DmgSuffix=qt4${Qt4API}.dmg
 
 # stupid special case...
 case "${Qt4Version}:${Qt4API}" in
@@ -56,6 +57,10 @@ case "${Qt4Version}:${Qt4API}" in
 	Qt4ConfigureOptions="${QtConfigureOptions} -opensource -silent -shared -fast -no-strip"
 	Qt4ConfigureOptions="${Qt4ConfigureOptions} -no-javascript-jit -no-pkg-config"
 	Qt4ConfigureOptions="${Qt4ConfigureOptions} -nomake examples -nomake demos -nomake docs -nomake tools"
+	Qt4DmgSuffix=qt5${Qt4API}.dmg
+	;;
+5.*)
+	Qt4DmgSuffix=qt5${Qt4API}.dmg
 	;;
 *)
 	Qt4ConfigureOptions="${Qt4ConfigureOptions} ${Qt4API}"
@@ -65,7 +70,7 @@ esac
 aspell_dictionaries="no"
 hunspell_dictionaries="yes"
 
-gettext_deployment="yes"
+gettext_deployment="no"
 aspell_deployment="yes"
 hunspell_deployment="yes"
 thesaurus_deployment="yes"
@@ -288,10 +293,14 @@ LyxAppPrefix="${LyxAppDir}.app"
 
 # don't change order here...
 case "${Qt4Version}" in
-5*)
+5.0.*|5.1.*)
 	QtLibraries="QtSvg QtXml QtPrintSupport QtWidgets QtGui QtNetwork QtConcurrent QtCore"
 	QtFrameworkVersion="5"
 	;;
+5*)
+	QtLibraries="QtSvg QtXml QtPrintSupport QtMacExtras QtWidgets QtGui QtNetwork QtConcurrent QtCore"
+	QtFrameworkVersion="5"
+	;;
 *)
 	QtLibraries="QtSvg QtXml QtGui QtNetwork QtCore"
 	QtFrameworkVersion="4"
@@ -326,7 +335,10 @@ case $SDKs in
 	exit 1
 	;;
 esac
-MYCFLAGS="-mmacosx-version-min=${MACOSX_DEPLOYMENT_TARGET} -D_XOPEN_SOURCE"
+MYCFLAGS="-mmacosx-version-min=${MACOSX_DEPLOYMENT_TARGET}"
+
+# pkg-config is not usable with this script
+export PKG_CONFIG=""
 
 # These variables define the identifiers of the
 # system (both Intel and PowerPC) to compile for.
@@ -607,7 +619,6 @@ build_lyx() {
 			export QT4_CORE_LIBS="-framework QtCore"
 			export QT4_FRONTEND_CFLAGS="-FQtGui"
 			export QT4_FRONTEND_LIBS="-framework QtGui"
-			export PKG_CONFIG=""
 			CPPFLAGS="${CPPFLAGS} -I${SDKROOT}/Library/Frameworks/QtCore.framework/Headers"
 			CPPFLAGS="${CPPFLAGS} -I${SDKROOT}/Library/Frameworks/QtGui.framework/Headers"
 		fi
@@ -671,7 +682,7 @@ Translations = translations
 EOF
 	if [ ! -d "${condir}/PlugIns" ]; then
 		mkdir -p "${condir}/PlugIns"
-		find "${source}/plugins" -name \*.dylib -print | while read libname ; do
+		find "${source}/plugins" -name \*.dylib -print | grep -v _debug.dylib | while read libname ; do
 			echo Copy plugin "${libname}"
 			dylib=$(basename "${libname}")
 			dirname=$(dirname "${libname}")
@@ -814,7 +825,7 @@ set_bundle_display_options() {
             set position of item "${LyxName}.app" to {$LYX_X_POSITION,$LYX_Y_POSITION}
             set position of item "Applications" to {$APP_X_POSITION,$APP_Y_POSITION}
             set background picture of the icon view options\
-					of container window to file "background.png" of folder "Pictures"
+               of container window to file "background.png" of folder "Pictures"
             set the bounds of the container window to {0, 0, $X_BOUNDS, $Y_BOUNDS}
             update without registering applications
             delay 5 -- sync
@@ -879,9 +890,9 @@ build_package() {
 	test -n "${DMGLocation}" && (
 		make_dmg "${DMGLocation}"
 		if [ -d "${QtInstallDir}/lib/QtCore.framework/Versions/${QtFrameworkVersion}" -a "yes" = "${qt4_deployment}" ]; then
-			rm -f "${DMGLocation}/${DMGNAME}+qt4.dmg"
-			echo move to "${DMGLocation}/${DMGNAME}+qt4${Qt4API}.dmg"
-			mv "${DMGLocation}/${DMGNAME}.dmg" "${DMGLocation}/${DMGNAME}+qt4${Qt4API}.dmg"
+			rm -f "${DMGLocation}/${DMGNAME}+${Qt4DmgSuffix}"
+			echo move to "${DMGLocation}/${DMGNAME}+${Qt4DmgSuffix}"
+			mv "${DMGLocation}/${DMGNAME}.dmg" "${DMGLocation}/${DMGNAME}+${Qt4DmgSuffix}"
 		fi
 	)
 }
