@@ -33,6 +33,7 @@ namespace lyx {
 
 
 Counter::Counter()
+	: initial_value_(0)
 {
 	reset();
 }
@@ -40,7 +41,7 @@ Counter::Counter()
 
 Counter::Counter(docstring const & mc, docstring const & ls,  
                 docstring const & lsa)
-	: master_(mc), labelstring_(ls), labelstringappendix_(lsa)
+	: initial_value_(0), master_(mc), labelstring_(ls), labelstringappendix_(lsa)
 {
 	reset();
 }
@@ -53,11 +54,13 @@ bool Counter::read(Lexer & lex)
 		CT_LABELSTRING,
 		CT_LABELSTRING_APPENDIX,
 		CT_PRETTYFORMAT,
+		CT_INITIALVALUE,
 		CT_END
 	};
 
 	LexerKeyword counterTags[] = {
 		{ "end", CT_END },
+	  { "initialvalue", CT_INITIALVALUE},
 		{ "labelstring", CT_LABELSTRING },
 		{ "labelstringappendix", CT_LABELSTRING_APPENDIX },
 		{ "prettyformat", CT_PRETTYFORMAT },
@@ -83,6 +86,18 @@ bool Counter::read(Lexer & lex)
 				if (master_ == "none")
 					master_.erase();
 				break;
+			case CT_INITIALVALUE:
+				lex.next();
+				initial_value_ = lex.getInteger();
+				// getInteger() returns -1 on error, and larger
+				// negative values do not make much sense.
+				// In the other case, we subtract one, since the
+				// counter will be incremented before its first use.
+				if (initial_value_ <= -1)
+					initial_value_ = 0;
+				else
+					initial_value_ -= 1;
+				break;
 			case CT_PRETTYFORMAT:
 				lex.next();
 				prettyformat_ = lex.getDocString();
@@ -109,6 +124,7 @@ bool Counter::read(Lexer & lex)
 	return getout;
 }
 
+
 void Counter::set(int v)
 {
 	value_ = v;
@@ -135,7 +151,7 @@ void Counter::step()
 
 void Counter::reset()
 {
-	value_ = 0;
+	value_ = initial_value_;
 }
 
 
@@ -166,7 +182,7 @@ Counter::StringMap & Counter::flatLabelStrings(bool in_appendix) const
 }
 
 
-Counters::Counters() : appendix_(false), subfloat_(false)
+Counters::Counters() : appendix_(false), subfloat_(false), longtable_(false)
 {
 	layout_stack_.push_back(0);
 	counter_stack_.push_back(from_ascii(""));
@@ -260,7 +276,7 @@ void Counters::step(docstring const & ctr, UpdateType utype)
 
 	it->second.step();
 	if (utype == OutputUpdate) {
-		LASSERT(!counter_stack_.empty(), /* */);
+		LBUFERR(!counter_stack_.empty());
 		counter_stack_.pop_back();
 		counter_stack_.push_back(ctr);
 	}
@@ -292,7 +308,7 @@ void Counters::reset()
 
 void Counters::reset(docstring const & match)
 {
-	LASSERT(!match.empty(), /**/);
+	LASSERT(!match.empty(), return);
 
 	CounterList::iterator it = counterList_.begin();
 	CounterList::iterator end = counterList_.end();
@@ -363,7 +379,6 @@ char hebrewCounter(int const n)
 }
 
 
-
 // On the special cases, see http://mathworld.wolfram.com/RomanNumerals.html
 // and for a list of roman numerals up to and including 3999, see 
 // http://www.research.att.com/~njas/sequences/a006968.txt. (Thanks to Joost
@@ -597,7 +612,7 @@ docstring Counters::prettyCounter(docstring const & name,
 
 docstring Counters::currentCounter() const
 { 
-	LASSERT(!counter_stack_.empty(), /* */);
+	LBUFERR(!counter_stack_.empty());
 	return counter_stack_.back(); 
 }
 
