@@ -19,8 +19,6 @@
 #include "FontEnums.h"
 #include "SpellChecker.h"
 
-#include "insets/InsetCode.h"
-
 #include "support/strfwd.h"
 #include "support/types.h"
 
@@ -154,7 +152,8 @@ public:
 	void setId(int id);
 
 	///
-	void addChangesToToc(DocIterator const & cdit, Buffer const & buf) const;
+	void addChangesToToc(DocIterator const & cdit, Buffer const & buf,
+	        bool output_active) const;
 	///
 	Language const * getParLanguage(BufferParams const &) const;
 	///
@@ -182,18 +181,19 @@ public:
 
 	/// Extract only the explicitly visible text (without any formatting),
 	/// descending into insets
-	docstring stringify(pos_type beg, pos_type end, int options, OutputParams & runparams) const;
+	docstring stringify(pos_type beg, pos_type end, int options,
+		OutputParams const & runparams) const;
 
 	///
 	void write(std::ostream &, BufferParams const &,
-		   depth_type & depth) const;
+		depth_type & depth) const;
 	///
 	void validate(LaTeXFeatures &) const;
 
 	/// \param force means: output even if layout.inpreamble is true.
 	void latex(BufferParams const &, Font const & outerfont, otexstream &,
-		   OutputParams const &, int start_pos = 0, int end_pos = -1,
-		   bool force = false) const;
+		OutputParams const &, int start_pos = 0, int end_pos = -1,
+		bool force = false) const;
 
 	/// Can we drop the standard paragraph wrapper?
 	bool emptyTag() const;
@@ -377,10 +377,6 @@ public:
 	///
 	void insertChar(pos_type pos, char_type c,
 			Font const &, Change const & change);
-	/// Insert \p inset at position \p pos with \p change traking status.
-	/// \return true if successful.
-	bool insertInset(pos_type pos, Inset * inset,
-			 Change const & change);
 	/// Insert \p inset at position \p pos with \p change traking status and
 	/// \p font.
 	/// \return true if successful.
