@@ -32,9 +32,12 @@
 
 #include "Buffer.h"
 #include "BufferList.h"
+#include "BufferParams.h"
 #include "BufferView.h"
 #include "CmdDef.h"
 #include "Color.h"
+#include "CutAndPaste.h"
+#include "ErrorList.h"
 #include "Font.h"
 #include "FuncRequest.h"
 #include "FuncStatus.h"
@@ -54,6 +57,8 @@
 #include "Thesaurus.h"
 #include "version.h"
 
+#include "insets/InsetText.h"
+
 #include "support/convert.h"
 #include "support/debug.h"
 #include "support/ExceptionMessage.h"
@@ -81,6 +86,7 @@
 #include <QByteArray>
 #include <QClipboard>
 #include <QDateTime>
+#include <QDesktopWidget>
 #include <QDir>
 #include <QEvent>
 #include <QEventLoop>
@@ -1914,7 +1920,16 @@ void GuiApplication::dispatch(FuncRequest const & cmd, DispatchResult & dr)
 		istringstream ss(argument);
 		Lexer lex;
 		lex.setStream(ss);
+		
+		// See #9236
+		// We need to make sure that, after we recreat the DocumentClass, 
+		// which we do in readHeader, we apply it to the document itself.
+		DocumentClassConstPtr olddc = defaults.params().documentClassPtr();
 		int const unknown_tokens = defaults.readHeader(lex);
+		DocumentClassConstPtr newdc = defaults.params().documentClassPtr();
+		ErrorList el;
+		InsetText & theinset = static_cast<InsetText &>(defaults.inset());
+		cap::switchBetweenClasses(olddc, newdc, theinset, el);
 
 		if (unknown_tokens != 0) {
 			lyxerr << "Warning in LFUN_BUFFER_SAVE_AS_DEFAULT!\n"
@@ -2197,13 +2212,37 @@ void GuiApplication::createView(QString const & geometry_arg, bool autoShow,
 #ifdef Q_WS_WIN
 		int x, y;
 		int w, h;
-		QRegExp re( "[=]*(?:([0-9]+)[xX]([0-9]+)){0,1}[ ]*(?:([+-][0-9]*)([+-][0-9]*)){0,1}" );
+		QChar sx, sy;
+		QRegExp re( "[=]*(?:([0-9]+)[xX]([0-9]+)){0,1}[ ]*(?:([+-][0-9]*)){0,1}(?:([+-][0-9]*)){0,1}" );
 		re.indexIn(geometry_arg);
 		w = re.cap(1).toInt();
 		h = re.cap(2).toInt();
 		x = re.cap(3).toInt();
 		y = re.cap(4).toInt();
+		sx = re.cap(3).isEmpty() ? '+' : re.cap(3).at(0);
+		sy = re.cap(4).isEmpty() ? '+' : re.cap(4).at(0);
+		// Set initial geometry such that we can get the frame size.
 		view->setGeometry(x, y, w, h);
+		int framewidth = view->geometry().x() - view->x();
+		int titleheight = view->geometry().y() - view->y();
+		// Negative displacements must be interpreted as distances
+		// from the right or bottom screen borders.
+		if (sx == '-' || sy == '-') {
+			QRect rec = QApplication::desktop()->screenGeometry();
+			if (sx == '-')
+				x += rec.width() - w - framewidth;
+			if (sy == '-')
+				y += rec.height() - h - titleheight;
+			view->setGeometry(x, y, w, h);
+		}
+		// Make sure that the left and top frame borders are visible.
+		if (view->x() < 0 || view->y() < 0) {
+			if (view->x() < 0)
+				x = framewidth;
+			if (view->y() < 0)
+				y = titleheight;
+			view->setGeometry(x, y, w, h);
+		}
 #endif
 	}
 	view->setFocus();
