@@ -23,6 +23,7 @@
 #include "insets/ExternalTemplate.h"
 #include "insets/InsetExternal.h"
 
+#include "graphics/epstools.h"
 #include "graphics/GraphicsCache.h"
 #include "graphics/GraphicsCacheItem.h"
 #include "graphics/GraphicsImage.h"
@@ -199,7 +200,7 @@ GuiExternal::GuiExternal(GuiView & lv)
 	i1 = external::TemplateManager::get().getTemplates().begin();
 	i2 = external::TemplateManager::get().getTemplates().end();
 	for (; i1 != i2; ++i1)
-		externalCO->addItem(qt_(i1->second.lyxName));
+		externalCO->addItem(qt_(i1->second.guiName));
 
 	// Fill the origins combo
 	for (size_t i = 0; i != sizeof(origins) / sizeof(origins[0]); ++i)
@@ -299,7 +300,7 @@ void GuiExternal::getbbClicked()
 	FileName const abs_file(support::makeAbsPath(filename, fromqstr(bufferFilePath())));
 
 	// try to get it from the file, if possible
-	string bb = readBB_from_PSFile(abs_file);
+	string bb = graphics::readBB_from_PSFile(abs_file);
 	if (bb.empty()) {
 		// we don't, so ask the Graphics Cache if it has loaded the file
 		int width = 0;
@@ -623,7 +624,14 @@ static QStringList templateFilters(QString const & template_name)
 	external::Template const * const et_ptr =
 		etm.getTemplateByName(fromqstr(template_name));
 
-	return fileFilters(et_ptr ? toqstr(et_ptr->fileRegExp) : QString());
+	string filter;
+	if (et_ptr && et_ptr->fileRegExp != "" && et_ptr->fileRegExp != "*") {
+		filter += et_ptr->guiName;
+		filter += " (";
+		filter += et_ptr->fileRegExp;
+		filter += ")";
+	}
+	return fileFilters(toqstr(filter));
 }
 
 
