@@ -189,6 +189,12 @@ bool InsetFloat::getStatus(Cursor & cur, FuncRequest const & cmd,
 			return true;
 		} else
 			return false;
+	
+	case LFUN_NEWLINE_INSERT:
+		if (params_.subfloat) {
+			flag.setEnabled(false);
+			return true;
+		}
 
 	default:
 		return InsetCollapsable::getStatus(cur, cmd, flag);
@@ -389,11 +395,11 @@ void InsetFloat::latex(otexstream & os, OutputParams const & runparams_in) const
 }
 
 
-int InsetFloat::plaintext(odocstream & os, OutputParams const & runparams) const
+int InsetFloat::plaintext(odocstringstream & os, OutputParams const & runparams, size_t max_length) const
 {
 	os << '[' << buffer().B_("float") << ' '
 		<< floatName(params_.type) << ":\n";
-	InsetText::plaintext(os, runparams);
+	InsetText::plaintext(os, runparams, max_length);
 	os << "\n]";
 
 	return PLAINTEXT_NEWLINE + 1; // one char on a separate line
@@ -470,6 +476,12 @@ void InsetFloat::setNewLabel()
 }
 
 
+bool InsetFloat::allowsCaptionVariation(std::string const & newtype) const
+{
+	return !params_.subfloat && newtype != "LongTableNoNumber";
+}
+
+
 docstring InsetFloat::getCaption(OutputParams const & runparams) const
 {
 	if (paragraphs().empty())
@@ -482,7 +494,7 @@ docstring InsetFloat::getCaption(OutputParams const & runparams) const
 	TexRow texrow;
 	odocstringstream ods;
 	otexstream os(ods, texrow);
-	ins->getOptArg(os, runparams);
+	ins->getArgs(os, runparams);
 	ods << '[';
 	odocstringstream odss;
 	otexstream oss(odss, texrow);
