@@ -74,7 +74,8 @@ private:
 	 */
 	void latex(otexstream &, OutputParams const &) const;
 	///
-	int plaintext(odocstream &, OutputParams const &) const;
+	int plaintext(odocstringstream & ods, OutputParams const & op,
+	              size_t max_length = INT_MAX) const;
 	///
 	int docbook(odocstream &, OutputParams const &) const;
 	///
@@ -85,6 +86,8 @@ private:
 	void validate(LaTeXFeatures & features) const;
 	/// returns LyX code associated with the inset. Used for TOC, ...)
 	InsetCode lyxCode() const { return GRAPHICS_CODE; }
+	///
+	docstring layoutName() const { return from_ascii("Graphics"); }
 	/// Get the inset parameters, used by the GUIndependent dialog.
 	InsetGraphicsParams const & params() const;
 	///
@@ -96,9 +99,9 @@ private:
 	///
 	bool getStatus(Cursor &, FuncRequest const &, FuncStatus &) const;
 	///
-	void addToToc(DocIterator const &) const;
+	void addToToc(DocIterator const & di, bool output_active) const;
 	///
-	docstring contextMenuName() const;
+	std::string contextMenuName() const;
 	/// Force inset into LTR environment if surroundings are RTL
 	bool forceLTR() const { return true; }
 	///
