@@ -101,13 +101,13 @@ void InsetMathFont::validate(LaTeXFeatures & features) const
 				|| (key_->name.length() == 6 && key_->name.substr(0, 4) == "text"))
 			features.require("amstext");
 		if (key_->name == "mathscr")
-			features.require("mathrsfs"); 
+			features.require("mathrsfs");
 		if (key_->name == "textipa")
 			features.require("tipa");
 		if (key_->name == "ce" || key_->name == "cf")
 			features.require("mhchem");
 	} else if (features.runparams().math_flavor == OutputParams::MathAsHTML) {
-		features.addPreambleSnippet("<style type=\"text/css\">\n"
+		features.addCSSSnippet(
 			"span.normal{font: normal normal normal inherit serif;}\n"
 			"span.fraktur{font: normal normal normal inherit cursive;}\n"
 			"span.bold{font: normal normal bold inherit serif;}\n"
@@ -115,8 +115,7 @@ void InsetMathFont::validate(LaTeXFeatures & features) const
 			"span.italic{font: italic normal normal inherit serif;}\n"
 			"span.sans{font: normal normal normal inherit sans-serif;}\n"
 			"span.monospace{font: normal normal normal inherit monospace;}\n"
-			"span.noun{font: normal small-caps normal inherit normal;}\n"
-			"</style>");
+			"span.noun{font: normal small-caps normal inherit normal;}");
 	}
 }
 
@@ -150,7 +149,7 @@ void InsetMathFont::htmlize(HtmlStream & os) const
 		variant = "monospace";
 	else if (tag == "textipa" || tag == "textsc" || tag == "noun")
 		variant = "noun";
-	
+
 	docstring const beg = (tag.size() < 4) ? from_ascii("") : tag.substr(0, 4);
 	if (!variant.empty()) {
 		os << MTag("span", "class='" + variant + "'")
@@ -189,7 +188,7 @@ void InsetMathFont::mathmlize(MathStream & os) const
 	else if (tag == "mathtt" || tag == "texttt")
 		variant = "monospace";
 	// no support at present for textipa, textsc, noun
-	
+
 	if (!variant.empty()) {
 		os << MTag("mstyle", "mathvariant='" + variant + "'")
 		   << cell(0)
