@@ -24,6 +24,7 @@
 #include "Text.h"
 
 #include "support/debug.h"
+#include "support/gettext.h"
 #include "support/lassert.h"
 
 #include <sstream>
@@ -44,16 +45,15 @@ MacroData::MacroData(Buffer * buf)
 	  redefinition_(false), type_(MacroTypeNewcommand)
 {}
 
-	
-	
+
 MacroData::MacroData(Buffer * buf, DocIterator const & pos)
 	: buffer_(buf), pos_(pos), queried_(false), numargs_(0),
 	  optionals_(0), lockCount_(0), redefinition_(false),
 	  type_(MacroTypeNewcommand)
 {
 }
-	
-	
+
+
 MacroData::MacroData(Buffer * buf, MathMacroTemplate const & macro)
 	: buffer_(buf), queried_(false), numargs_(0), optionals_(0), lockCount_(0),
 	  redefinition_(false), type_(MacroTypeNewcommand)
@@ -99,7 +99,7 @@ size_t MacroData::optionals() const
 }
 
 
-vector<docstring> const &  MacroData::defaults() const
+vector<docstring> const & MacroData::defaults() const
 {
 	updateData();
 	return defaults_;
@@ -109,7 +109,7 @@ vector<docstring> const &  MacroData::defaults() const
 void MacroData::unlock() const
 {
 	--lockCount_;
-	LASSERT(lockCount_ >= 0, /**/);
+	LASSERT(lockCount_ >= 0, lockCount_ = 0);
 }
 
 
@@ -125,7 +125,7 @@ void MacroData::queryData(MathMacroTemplate const & macro) const
 	redefinition_ = macro.redefinition();
 	type_ = macro.type();
 	optionals_ = macro.numOptionals();
-	
+
 	macro.getDefaults(defaults_);
 }
 
@@ -135,22 +135,22 @@ void MacroData::updateData() const
 	if (queried_)
 		return;
 
-	LASSERT(buffer_ != 0, /**/);
-	
+	LBUFERR(buffer_);
+
 	// Try to fix position DocIterator. Should not do anything in theory.
 	pos_.fixIfBroken();
-	
+
 	// find macro template
 	Inset * inset = pos_.nextInset();
 	if (inset == 0 || inset->lyxCode() != MATHMACRO_CODE) {
 		lyxerr << "BUG: No macro template found by MacroData" << endl;
 		return;
 	}
-	
+
 	// query the data from the macro template
-	queryData(static_cast<MathMacroTemplate const &>(*inset));	
+	queryData(static_cast<MathMacroTemplate const &>(*inset));
 }
-	
+
 
 int MacroData::write(odocstream & os, bool overwriteRedefinition) const
 {
@@ -162,7 +162,7 @@ int MacroData::write(odocstream & os, bool overwriteRedefinition) const
 		lyxerr << "BUG: No macro template found by MacroData" << endl;
 		return 0;
 	}
-		
+
 	// output template
 	MathMacroTemplate const & tmpl =
 		static_cast<MathMacroTemplate const &>(*inset);
@@ -191,20 +191,28 @@ MacroData const * MacroTable::get(docstring const & name) const
 }
 
 
-void MacroTable::insert(docstring const & name, MacroData const & data)
+MacroTable::iterator
+MacroTable::insert(docstring const & name, MacroData const & data)
 {
 	//lyxerr << "MacroTable::insert: " << to_utf8(name) << endl;
-	operator[](name) = data;
+	iterator it = find(name);
+	if (it == end())
+		it = map<docstring, MacroData>::insert(
+				make_pair(name, data)).first;
+	else
+		it->second = data;
+	return it;
 }
 
 
-void MacroTable::insert(Buffer * buf, docstring const & def, string const & requires)
+MacroTable::iterator
+MacroTable::insert(Buffer * buf, docstring const & def, string const & requires)
 {
 	//lyxerr << "MacroTable::insert, def: " << to_utf8(def) << endl;
 	MathMacroTemplate mac(buf, def);
 	MacroData data(buf, mac);
 	data.requires() = requires;
-	insert(mac.name(), data);
+	return insert(mac.name(), data);
 }
 
 
