@@ -465,7 +465,7 @@ docstring MathMacroTemplate::name() const
 }
 
 
-void MathMacroTemplate::updateToContext(MacroContext const & mc) const
+void MathMacroTemplate::updateToContext(MacroContext const & mc)
 {
 	redefinition_ = mc.get(name()) != 0;
 }
@@ -551,13 +551,9 @@ void MathMacroTemplate::metrics(MetricsInfo & mi, Dimension & dim) const
 
 	// valid macro?
 	MacroData const * macro = 0;
-	if (validName()) {
+	if (validName())
 		macro = mi.macrocontext.get(name());
 
-		// updateToContext() - avoids another lookup
-		redefinition_ = macro != 0;
-	}
-
 	// update look?
 	int argsInDef = maxArgumentInDefinition();
 	if (lookOutdated_ || argsInDef != argsInLook_) {
@@ -849,7 +845,7 @@ void fixMacroInstances(Cursor & cur, DocIterator const & inset_pos,
 		for (; sit != end; ++sit) {
 			InsetMathHull * inset_hull =
 				sit->nextInset()->asInsetMath()->asHullInset();
-			LASSERT(inset_hull, /**/);
+			LBUFERR(inset_hull);
 			inset_hull->reloadPreview(*sit);
 		}
 		cur.screenUpdateFlags(Update::Force);
@@ -997,7 +993,7 @@ void MathMacroTemplate::doDispatch(Cursor & cur, FuncRequest & cmd)
 			commitEditChanges(cur, cur);
 			cur.recordUndoFullDocument();
 			size_t pos = numargs_;
-			if (arg.size() != 0)
+			if (!arg.empty())
 				pos = (size_t)convert<int>(arg) - 1; // it is checked for >=0 in getStatus
 			insertParameter(cur, cur, pos);
 		}
@@ -1009,7 +1005,7 @@ void MathMacroTemplate::doDispatch(Cursor & cur, FuncRequest & cmd)
 			commitEditChanges(cur, cur);
 			cur.recordUndoFullDocument();
 			size_t pos = numargs_ - 1;
-			if (arg.size() != 0)
+			if (!arg.empty())
 				pos = (size_t)convert<int>(arg) - 1; // it is checked for >=0 in getStatus
 			removeParameter(cur, cur, pos);
 		}
@@ -1083,7 +1079,7 @@ bool MathMacroTemplate::getStatus(Cursor & /*cur*/, FuncRequest const & cmd,
 	switch (cmd.action()) {
 		case LFUN_MATH_MACRO_ADD_PARAM: {
 			int num = numargs_ + 1;
-			if (arg.size() != 0)
+			if (!arg.empty())
 				num = convert<int>(arg);
 			bool on = (num >= optionals_
 				   && numargs_ < 9 && num <= numargs_ + 1);
@@ -1098,7 +1094,7 @@ bool MathMacroTemplate::getStatus(Cursor & /*cur*/, FuncRequest const & cmd,
 		case LFUN_MATH_MACRO_REMOVE_GREEDY_PARAM:
 		case LFUN_MATH_MACRO_REMOVE_PARAM: {
 			int num = numargs_;
-			if (arg.size() != 0)
+			if (!arg.empty())
 				num = convert<int>(arg);
 			flag.setEnabled(num >= 1 && num <= numargs_);
 			break;
@@ -1255,8 +1251,8 @@ docstring MathMacroTemplate::xhtml(XHTMLStream &, OutputParams const &) const
 	return docstring();
 }
 
-int MathMacroTemplate::plaintext(odocstream & os,
-				 OutputParams const &) const
+int MathMacroTemplate::plaintext(odocstringstream & os,
+				 OutputParams const &, size_t) const
 {
 	static docstring const str = '[' + buffer().B_("math macro") + ']';
 
@@ -1269,8 +1265,7 @@ bool MathMacroTemplate::validName() const
 {
 	docstring n = name();
 
-	// empty name?
-	if (n.size() == 0)
+	if (n.empty())
 		return false;
 
 	// converting back and force doesn't swallow anything?
@@ -1319,7 +1314,7 @@ bool MathMacroTemplate::fixNameAndCheckIfValid()
 	}
 
 	// now it should be valid if anything in the name survived
-	return data.size() > 0;
+	return !data.empty();
 }
 
 	
@@ -1370,9 +1365,9 @@ void MathMacroTemplate::infoize(odocstream & os) const
 }
 
 
-docstring MathMacroTemplate::contextMenuName() const
+string MathMacroTemplate::contextMenuName() const
 {
-	return from_ascii("context-math-macro-definition");
+	return "context-math-macro-definition";
 }
 
 } // namespace lyx
