@@ -42,6 +42,7 @@ following hack as starting point to write some macros:
 #include "InsetMathArray.h"
 #include "InsetMathBig.h"
 #include "InsetMathBrace.h"
+#include "InsetMathCancelto.h"
 #include "InsetMathChar.h"
 #include "InsetMathColor.h"
 #include "InsetMathComment.h"
@@ -55,9 +56,11 @@ following hack as starting point to write some macros:
 #include "InsetMathRef.h"
 #include "InsetMathRoot.h"
 #include "InsetMathScript.h"
+#include "InsetMathSideset.h"
 #include "InsetMathSpace.h"
 #include "InsetMathSplit.h"
 #include "InsetMathSqrt.h"
+#include "InsetMathStackrel.h"
 #include "InsetMathString.h"
 #include "InsetMathTabular.h"
 #include "MathMacroTemplate.h"
@@ -333,9 +336,9 @@ public:
 	///
 	char_type character() const { return char_; }
 	///
-	docstring asString() const { return cs_.size() ? cs_ : docstring(1, char_); }
+	docstring asString() const { return !cs_.empty() ? cs_ : docstring(1, char_); }
 	///
-	docstring asInput() const { return cs_.size() ? '\\' + cs_ : docstring(1, char_); }
+	docstring asInput() const { return !cs_.empty() ? '\\' + cs_ : docstring(1, char_); }
 
 private:
 	///
@@ -349,7 +352,7 @@ private:
 
 ostream & operator<<(ostream & os, Token const & t)
 {
-	if (t.cs().size()) {
+	if (!t.cs().empty()) {
 		docstring const & cs = t.cs();
 		// FIXME: For some strange reason, the stream operator instanciate
 		// a new Token before outputting the contents of t.cs().
@@ -858,7 +861,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					} else {
 						// This is not an outer hull and display math is
 						// not allowed inside text mode environments.
-						error("bad math environment");
+						error("bad math environment $$");
 						break;
 					}
 				} else {
@@ -916,7 +919,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		}
 
 		else if (t.cat() == catActive)
-			cell->push_back(MathAtom(new InsetMathChar(t.character())));
+			cell->push_back(MathAtom(new InsetMathSpace(string(1, t.character()), "")));
 
 		else if (t.cat() == catBegin) {
 			MathData ar;
@@ -950,7 +953,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			bool up = (t.cat() == catSuper);
 			// we need no new script inset if the last thing was a scriptinset,
 			// which has that script already not the same script already
-			if (!cell->size())
+			if (cell->empty())
 				cell->push_back(MathAtom(new InsetMathScript(buf, up)));
 			else if (cell->back()->asScriptInset() &&
 					!cell->back()->asScriptInset()->has(up))
@@ -1017,7 +1020,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				Token const & t = getToken();
 				if (t.cat() == catNewline)
 					break;
-				s += t.asString();
+				s += t.asInput();
 			}
 			cell->push_back(MathAtom(new InsetMathComment(buf, s)));
 			skipSpaces();
@@ -1028,7 +1031,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		//
 
 		else if (t.cs() == "lyxlock") {
-			if (cell->size())
+			if (!cell->empty())
 				cell->back().nucleus()->lock(true);
 		}
 
@@ -1237,13 +1240,19 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		}
 
 		else if (t.cs() == "(") {
-			cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
-			parse(cell->back().nucleus()->cell(0), FLAG_SIMPLE2, InsetMath::MATH_MODE);
+			if (mode == InsetMath::UNDECIDED_MODE) {
+				cell->push_back(MathAtom(new InsetMathHull(buf, hullSimple)));
+				parse2(cell->back(), FLAG_SIMPLE2, InsetMath::MATH_MODE, false);
+			} else {
+				// Don't create nested math hulls (bug #5392)
+				cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
+				parse(cell->back().nucleus()->cell(0), FLAG_SIMPLE2, InsetMath::MATH_MODE);
+			}
 		}
 
 		else if (t.cs() == "[") {
 			if (mode != InsetMath::UNDECIDED_MODE) {
-				error("bad math environment");
+				error("bad math environment [");
 				break;
 			}
 			cell->push_back(MathAtom(new InsetMathHull(buf, hullEquation)));
@@ -1376,7 +1385,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 		else if (t.cs() == "sqrt") {
 			MathData ar;
 			parse(ar, FLAG_OPTION, mode);
-			if (ar.size()) {
+			if (!ar.empty()) {
 				cell->push_back(MathAtom(new InsetMathRoot(buf)));
 				cell->back().nucleus()->cell(0) = ar;
 				parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
@@ -1386,11 +1395,19 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 		}
 
+		else if (t.cs() == "cancelto") {
+			MathData ar;
+			parse(ar, FLAG_ITEM, mode);
+				cell->push_back(MathAtom(new InsetMathCancelto(buf)));
+				cell->back().nucleus()->cell(1) = ar;
+				parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
+		}
+
 		else if (t.cs() == "unit") {
 			// Allowed formats \unit[val]{unit}
 			MathData ar;
 			parse(ar, FLAG_OPTION, mode);
-			if (ar.size()) {
+			if (!ar.empty()) {
 				cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::UNIT)));
 				cell->back().nucleus()->cell(0) = ar;
 				parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
@@ -1404,7 +1421,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			// Here allowed formats are \unitfrac[val]{num}{denom}
 			MathData ar;
 			parse(ar, FLAG_OPTION, mode);
-			if (ar.size()) {
+			if (!ar.empty()) {
 				cell->push_back(MathAtom(new InsetMathFrac(buf, InsetMathFrac::UNITFRAC, 3)));
 				cell->back().nucleus()->cell(2) = ar;
 			} else {
@@ -1432,6 +1449,45 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
 		}
 
+		else if (t.cs() == "sideset") {
+			// Here allowed formats are \sideset{_{bl}^{tl}}{_{br}^{tr}}{operator}
+			MathData ar[2];
+			InsetMathScript * script[2] = {0, 0};
+			for (int i = 0; i < 2; ++i) {
+				parse(ar[i], FLAG_ITEM, mode);
+				if (ar[i].size() == 1)
+					script[i] = ar[i][0].nucleus()->asScriptInset();
+			}
+			bool const hasscript[2] = {script[0] ? true : false, script[1] ? true : false};
+			cell->push_back(MathAtom(new InsetMathSideset(buf, hasscript[0], hasscript[1])));
+			if (hasscript[0]) {
+				if (script[0]->hasDown())
+					cell->back().nucleus()->cell(1) = script[0]->down();
+				if (script[0]->hasUp())
+					cell->back().nucleus()->cell(2) = script[0]->up();
+			} else
+				cell->back().nucleus()->cell(1) = ar[0];
+			if (hasscript[1]) {
+				if (script[1]->hasDown())
+					cell->back().nucleus()->cell(2 + hasscript[0]) = script[1]->down();
+				if (script[1]->hasUp())
+					cell->back().nucleus()->cell(3 + hasscript[0]) = script[1]->up();
+			} else
+				cell->back().nucleus()->cell(2 + hasscript[0]) = ar[1];
+			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
+		}
+
+		else if (t.cs() == "stackrel") {
+			// Here allowed formats are \stackrel[subscript]{superscript}{operator}
+			MathData ar;
+			parse(ar, FLAG_OPTION, mode);
+			cell->push_back(MathAtom(new InsetMathStackrel(buf, !ar.empty())));
+			if (!ar.empty())
+				cell->back().nucleus()->cell(2) = ar;
+			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
+			parse(cell->back().nucleus()->cell(1), FLAG_ITEM, mode);
+		}
+
 		else if (t.cs() == "xrightarrow" || t.cs() == "xleftarrow") {
 			cell->push_back(createInsetMath(t.cs(), buf));
 			parse(cell->back().nucleus()->cell(1), FLAG_OPTION, mode);
@@ -1517,14 +1573,20 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 
 			else if (name == "math") {
-				cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
-				parse(cell->back().nucleus()->cell(0), FLAG_END, InsetMath::MATH_MODE);
+				if (mode == InsetMath::UNDECIDED_MODE) {
+					cell->push_back(MathAtom(new InsetMathHull(buf, hullSimple)));
+					parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, false);
+				} else {
+					// Don't create nested math hulls (bug #5392)
+					cell->push_back(MathAtom(new InsetMathEnsureMath(buf)));
+					parse(cell->back().nucleus()->cell(0), FLAG_END, InsetMath::MATH_MODE);
+				}
 			}
 
 			else if (name == "equation" || name == "equation*"
 					|| name == "displaymath") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				cell->push_back(MathAtom(new InsetMathHull(buf, hullEquation)));
@@ -1533,7 +1595,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else if (name == "eqnarray" || name == "eqnarray*") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				cell->push_back(MathAtom(new InsetMathHull(buf, hullEqnArray)));
@@ -1541,17 +1603,19 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 
 			else if (name == "align" || name == "align*") {
-				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
-					break;
+				if (mode == InsetMath::UNDECIDED_MODE) {
+					cell->push_back(MathAtom(new InsetMathHull(buf, hullAlign)));
+					parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
+				} else {
+					cell->push_back(MathAtom(new InsetMathSplit(buf, name,
+							'c', !stared(name))));
+					parse2(cell->back(), FLAG_END, mode, !stared(name));
 				}
-				cell->push_back(MathAtom(new InsetMathHull(buf, hullAlign)));
-				parse2(cell->back(), FLAG_END, InsetMath::MATH_MODE, !stared(name));
 			}
 
 			else if (name == "flalign" || name == "flalign*") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				cell->push_back(MathAtom(new InsetMathHull(buf, hullFlAlign)));
@@ -1560,7 +1624,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else if (name == "alignat" || name == "alignat*") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				// ignore this for a while
@@ -1571,7 +1635,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else if (name == "xalignat" || name == "xalignat*") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				// ignore this for a while
@@ -1582,7 +1646,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else if (name == "xxalignat") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				// ignore this for a while
@@ -1593,7 +1657,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else if (name == "multline" || name == "multline*") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				cell->push_back(MathAtom(new InsetMathHull(buf, hullMultline)));
@@ -1602,7 +1666,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 
 			else if (name == "gather" || name == "gather*") {
 				if (mode != InsetMath::UNDECIDED_MODE) {
-					error("bad math environment");
+					error("bad math environment " + name);
 					break;
 				}
 				cell->push_back(MathAtom(new InsetMathHull(buf, hullGather)));
@@ -1760,12 +1824,17 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			parse(cell->back().nucleus()->cell(0), FLAG_ITEM, InsetMath::TEXT_MODE);
 		}
 
-		else if (t.cs() == "hspace" && nextToken().character() != '*') {
+		else if (t.cs() == "hspace") {
+			bool const prot =  nextToken().character() == '*';
+			if (prot)
+				getToken();
 			docstring const name = t.cs();
 			docstring const arg = parse_verbatim_item();
 			Length length;
-			if (isValidLength(to_utf8(arg), &length))
-				cell->push_back(MathAtom(new InsetMathSpace(length)));
+			if (prot && arg == "\\fill")
+				cell->push_back(MathAtom(new InsetMathSpace("hspace*{\\fill}", "")));
+			else if (isValidLength(to_utf8(arg), &length))
+				cell->push_back(MathAtom(new InsetMathSpace(length, prot)));
 			else {
 				// Since the Length class cannot use length variables
 				// we must not create an InsetMathSpace.
@@ -1776,6 +1845,27 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 			}
 		}
 
+		else if (t.cs() == "smash") {
+			skipSpaces();
+			if (nextToken().asInput() == "[") {
+				// Since the phantom inset cannot handle optional arguments
+				// we must not create an InsetMathPhantom (bug 8967).
+				docstring const opt = parse_verbatim_option();
+				docstring const arg = parse_verbatim_item();
+				cell->push_back(MathAtom(new MathMacro(buf, t.cs())));
+				MathData ar;
+				mathed_parse_cell(ar, '[' + opt + ']', mode_);
+				cell->append(ar);
+				ar = MathData();
+				mathed_parse_cell(ar, '{' + arg + '}', mode_);
+				cell->append(ar);
+			}
+			else {
+				cell->push_back(createInsetMath(t.cs(), buf));
+				parse(cell->back().nucleus()->cell(0), FLAG_ITEM, mode);
+			}
+		}
+
 #if 0
 		else if (t.cs() == "infer") {
 			MathData ar;
@@ -1823,25 +1913,28 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 				error("'}' expected in \\" + t.cs());
 				return success_;
 			}
+			bool termination;
 			docstring rem;
 			do {
-				cmd = Encodings::fromLaTeXCommand(cmd, rem);
+				cmd = Encodings::fromLaTeXCommand(cmd,
+					Encodings::MATH_CMD | Encodings::TEXT_CMD,
+					termination, rem);
 				for (size_t i = 0; i < cmd.size(); ++i)
 					cell->push_back(MathAtom(new InsetMathChar(cmd[i])));
-				if (rem.size()) {
+				if (!rem.empty()) {
 					char_type c = rem[0];
 					cell->push_back(MathAtom(new InsetMathChar(c)));
 					cmd = rem.substr(1);
 					rem.clear();
 				} else
 					cmd.clear();
-			} while (cmd.size());
+			} while (!cmd.empty());
 		}
 
-		else if (t.cs().size()) {
+		else if (!t.cs().empty()) {
 			bool const no_mhchem =
 				(t.cs() == "ce" || t.cs() == "cf")
-				&& buf && buf->params().use_mhchem ==
+				&& buf && buf->params().use_package("mhchem") ==
 						BufferParams::package_off;
 
 			bool const is_user_macro = no_mhchem ||
@@ -1917,8 +2010,10 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 						}
 					}
 					bool is_combining;
-					char_type c =
-						Encodings::fromLaTeXCommand(cmd, is_combining);
+					bool termination;
+					char_type c = Encodings::fromLaTeXCommand(cmd,
+						Encodings::MATH_CMD | Encodings::TEXT_CMD,
+						is_combining, termination);
 					if (is_combining) {
 						if (cat == catLetter)
 							cmd += '{';
@@ -1926,9 +2021,26 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 						++num_tokens;
 						if (cat == catLetter)
 							cmd += '}';
-						c = Encodings::fromLaTeXCommand(cmd, is_combining);
+						c = Encodings::fromLaTeXCommand(cmd,
+							Encodings::MATH_CMD | Encodings::TEXT_CMD,
+							is_combining, termination);
 					}
 					if (c) {
+						if (termination) {
+							if (nextToken().cat() == catBegin) {
+								getToken();
+								if (nextToken().cat() == catEnd) {
+									getToken();
+									num_tokens += 2;
+								} else
+									putback();
+							} else {
+								while (nextToken().cat() == catSpace) {
+									getToken();
+									++num_tokens;
+								}
+							}
+						}
 						is_unicode_symbol = true;
 						cell->push_back(MathAtom(new InsetMathChar(c)));
 					} else {
@@ -1950,7 +2062,7 @@ bool Parser::parse1(InsetMathGrid & grid, unsigned flags,
 					// this fails on \bigg[...\bigg]
 					//MathData opt;
 					//parse(opt, FLAG_OPTION, InsetMath::VERBATIM_MODE);
-					//if (opt.size()) {
+					//if (!opt.empty()) {
 					//	start = 1;
 					//	at.nucleus()->cell(0) = opt;
 					//}
