@@ -13,7 +13,8 @@
 
 #include "MathSupport.h"
 
-#include "InsetMath.h"
+#include "InsetMathFont.h"
+#include "InsetMathSymbol.h"
 #include "MathData.h"
 #include "MathParser.h"
 #include "MathStream.h"
@@ -137,7 +138,7 @@ double const udarrow[] = {
 	0.015, 0.25,  0.5, 0.05, 0.95, 0.25,
 	2, 3,
 	0.015, 0.75,  0.5, 0.95, 0.95, 0.75,
-	1, 0.5, 0.2,  0.5, 0.8,
+	1, 0.5, 0.1,  0.5, 0.9,
 	0
 };
 
@@ -160,6 +161,15 @@ double const brack[] = {
 };
 
 
+double const dbrack[] = {
+	2, 4,
+	0.95, 0.05,  0.05, 0.05,  0.05, 0.95,  0.95, 0.95,
+	2, 2,
+	0.50, 0.05,  0.50, 0.95,
+	0
+};
+
+
 double const corner[] = {
 	2, 3,
 	0.95, 0.05,  0.05, 0.05,  0.05, 0.95,
@@ -299,11 +309,17 @@ named_deco_struct deco_table[] = {
 	{"rbrace",         brace,      2 },
 	{"[",              brack,      0 },
 	{"]",              brack,      2 },
+	{"llbracket",      dbrack,     0 },
+	{"rrbracket",      dbrack,     2 },
 	{"|",              vert,       0 },
 	{"/",              slash,      0 },
 	{"slash",          slash,      0 },
 	{"vert",           vert,       0 },
+	{"lvert",          vert,       0 },
+	{"rvert",          vert,       0 },
 	{"Vert",           Vert,       0 },
+	{"lVert",          Vert,       0 },
+	{"rVert",          Vert,       0 },
 	{"'",              slash,      1 },
 	{"<",              angle,      0 },
 	{">",              angle,      2 },
@@ -559,7 +575,7 @@ fontinfo fontinfos[] = {
 	{"mathit",        inh_family, inh_series,
 			  ITALIC_SHAPE, Color_math},
 	{"mathscr",       RSFS_FAMILY, inh_series,
-	          inh_shape, Color_math}, 
+			  inh_shape, Color_math},
 	{"cmex",          CMEX_FAMILY, inh_series,
 			  inh_shape, Color_math},
 	{"cmm",           CMM_FAMILY, inh_series,
@@ -574,6 +590,8 @@ fontinfo fontinfos[] = {
 			  inh_shape, Color_math},
 	{"msb",           MSB_FAMILY, inh_series,
 			  inh_shape, Color_math},
+	{"stmry",         STMARY_FAMILY, inh_series,
+			  inh_shape, Color_math},
 	{"wasy",          WASY_FAMILY, inh_series,
 			  inh_shape, Color_math},
 	{"esint",         ESINT_FAMILY, inh_series,
@@ -723,6 +741,27 @@ void augmentFont(FontInfo & font, docstring const & name)
 }
 
 
+bool isAlphaSymbol(MathAtom const & at)
+{
+	if (at->asCharInset() ||
+	    (at->asSymbolInset() &&
+	     at->asSymbolInset()->isOrdAlpha()))
+		return true;
+
+	if (at->asFontInset()) {
+		MathData const & ar = at->asFontInset()->cell(0);
+		for (size_t i = 0; i < ar.size(); ++i) {
+			if (!(ar[i]->asCharInset() ||
+			      (ar[i]->asSymbolInset() &&
+			       ar[i]->asSymbolInset()->isOrdAlpha())))
+				return false;
+		}
+		return true;
+	}
+	return false;
+}
+
+
 docstring asString(MathData const & ar)
 {
 	odocstringstream os;
