@@ -32,7 +32,7 @@
 #include "support/os.h"
 #include "support/Messages.h"
 #include "support/Package.h"
-#include "support/Path.h"
+#include "support/PathChanger.h"
 #include "support/Systemcall.h"
 #include "support/qstring_helpers.h"
 
@@ -101,13 +101,13 @@ string const latex_path(string const & original_path,
 		// We can't use '"' because " is sometimes active (e.g. if
 		// babel is loaded with the "german" option)
 		if (extension == EXCLUDE_EXTENSION) {
-			// ChangeExtension calls os::internal_path internally
+			// changeExtension calls os::internal_path internally
 			// so don't use it to remove the extension.
 			string const ext = getExtension(path);
 			string const base = ext.empty() ?
 				path :
 				path.substr(0, path.length() - ext.length() - 1);
-			// ChangeExtension calls os::internal_path internally
+			// changeExtension calls os::internal_path internally
 			// so don't use it to re-add the extension.
 			path = "\\string\"" + base + "\\string\"." + ext;
 		} else {
@@ -115,7 +115,18 @@ string const latex_path(string const & original_path,
 		}
 	}
 
-	return dots == ESCAPE_DOTS ? subst(path, ".", "\\lyxdot ") : path;
+	if (dots != ESCAPE_DOTS)
+		return path;
+
+	// Replace dots with the lyxdot macro, but only in the file name,
+	// not the directory part.
+	// addName etc call os::internal_path internally
+	// so don't use them for path manipulation
+	// The directory separator is always '/' for LaTeX.
+	string::size_type pos = path.rfind('/');
+	if (pos == string::npos)
+		return subst(path, ".", "\\lyxdot ");
+	return path.substr(0, pos) + subst(path.substr(pos), ".", "\\lyxdot ");
 }
 
 
@@ -1006,59 +1017,6 @@ FileName const findtexfile(string const & fil, string const & /*format*/)
 }
 
 
-void readBB_lyxerrMessage(FileName const & file, bool & zipped,
-	string const & message)
-{
-	LYXERR(Debug::GRAPHICS, "[readBB_from_PSFile] " << message);
-	// FIXME: Why is this func deleting a file? (Lgb)
-	if (zipped)
-		file.removeFile();
-}
-
-
-string const readBB_from_PSFile(FileName const & file)
-{
-	// in a (e)ps-file it's an entry like %%BoundingBox:23 45 321 345
-	// It seems that every command in the header has an own line,
-	// getline() should work for all files.
-	// On the other hand some plot programs write the bb at the
-	// end of the file. Than we have in the header:
-	// %%BoundingBox: (atend)
-	// In this case we must check the end.
-	bool zipped = file.isZippedFile();
-	FileName const file_ = zipped ? unzipFile(file) : file;
-	string const format = file_.guessFormatFromContents();
-
-	if (format != "eps" && format != "ps") {
-		readBB_lyxerrMessage(file_, zipped,"no(e)ps-format");
-		return string();
-	}
-
-	static lyx::regex bbox_re(
-		"^%%BoundingBox:\\s*([-]*[[:digit:]]+)\\s+([-]*[[:digit:]]+)\\s+([-]*[[:digit:]]+)\\s+([-]*[[:digit:]]+)");
-	ifstream is(file_.toFilesystemEncoding().c_str());
-	while (is) {
-		string s;
-		getline(is,s);
-		lyx::smatch what;
-		if (regex_match(s, what, bbox_re)) {
-			// Our callers expect the tokens in the string
-			// separated by single spaces.
-			// FIXME: change return type from string to something
-			// sensible
-			ostringstream os;
-			os << what.str(1) << ' ' << what.str(2) << ' '
-			   << what.str(3) << ' ' << what.str(4);
-			string const bb = os.str();
-			readBB_lyxerrMessage(file_, zipped, bb);
-			return bb;
-		}
-	}
-	readBB_lyxerrMessage(file_, zipped, "no bb found");
-	return string();
-}
-
-
 int compare_timestamps(FileName const & file1, FileName const & file2)
 {
 	// If the original is newer than the copy, then copy the original
@@ -1106,6 +1064,29 @@ bool prefs2prefs(FileName const & filename, FileName const & tempfile, bool lfun
 	return true;
 }
 
+int fileLock(const char * lock_file)
+{
+	int fd = -1;
+#if defined(HAVE_LOCKF)
+	fd = open(lock_file, O_CREAT|O_APPEND|O_SYNC|O_RDWR, 0666);
+	if (lockf(fd, F_LOCK, 0) != 0) {
+		close(fd);
+		return(-1);
+	}
+#endif
+	return(fd);
+}
+
+void fileUnlock(int fd, const char * /* lock_file*/)
+{
+#if defined(HAVE_LOCKF)
+	if (fd >= 0) {
+		if (lockf(fd, F_ULOCK, 0))
+			LYXERR0("Can't unlock the file.");
+		close(fd);
+	}
+#endif
+}
 
 } //namespace support
 } // namespace lyx
