@@ -46,9 +46,11 @@ docstring const user_name()
 	return from_local8bit(name);
 #else
 	struct passwd * pw = getpwuid(geteuid());
-	LASSERT(pw, /**/);
+	LASSERT(pw, return docstring());
 
-	string name = pw->pw_gecos;
+	const string gecos = pw->pw_gecos;
+	const size_t pos = gecos.find(",");
+	string name = gecos.substr(0, pos);
 	if (name.empty())
 		name = pw->pw_name;
 	return from_local8bit(name);
