@@ -3,7 +3,7 @@
  * This file is part of LyX, the document processor.
  * Licence details can be found in the file COPYING.
  *
- * \author André Pönitz 
+ * \author André Pönitz
  *
  * Full author contact details are available in file CREDITS.
  */
@@ -12,55 +12,25 @@
 
 #include "Encoding.h"
 #include "Parser.h"
+#include "support/foreach.h"
+#include "support/lstrings.h"
 #include "support/textutils.h"
 
 #include <iostream>
 
 using namespace std;
+using namespace lyx::support;
 
 namespace lyx {
 
 namespace {
 
-CatCode theCatcode[256];
-
-void catInit()
-{
-	static bool init_done = false;
-	if (init_done) 
-		return;
-	init_done = true;
-
-	fill(theCatcode, theCatcode + 256, catOther);
-	fill(theCatcode + 'a', theCatcode + 'z' + 1, catLetter);
-	fill(theCatcode + 'A', theCatcode + 'Z' + 1, catLetter);
-
-	theCatcode[int('\\')] = catEscape;
-	theCatcode[int('{')]  = catBegin;
-	theCatcode[int('}')]  = catEnd;
-	theCatcode[int('$')]  = catMath;
-	theCatcode[int('&')]  = catAlign;
-	theCatcode[int('\n')] = catNewline;
-	theCatcode[int('#')]  = catParameter;
-	theCatcode[int('^')]  = catSuper;
-	theCatcode[int('_')]  = catSub;
-	theCatcode[0x7f]      = catIgnore;
-	theCatcode[int(' ')]  = catSpace;
-	theCatcode[int('\t')] = catSpace;
-	theCatcode[int('\r')] = catNewline;
-	theCatcode[int('~')]  = catActive;
-	theCatcode[int('%')]  = catComment;
-
-	// This is wrong!
-	theCatcode[int('@')]  = catLetter;
-}
-
 /*!
  * Translate a line ending to '\n'.
  * \p c must have catcode catNewline, and it must be the last character read
  * from \p is.
  */
-char_type getNewline(idocstream & is, char_type c)
+char_type getNewline(iparserdocstream & is, char_type c)
 {
 	// we have to handle 3 different line endings:
 	// - UNIX (\n)
@@ -79,16 +49,8 @@ char_type getNewline(idocstream & is, char_type c)
 	return c;
 }
 
-CatCode catcode(char_type c)
-{
-	if (c < 256)
-		return theCatcode[(unsigned char)c];
-	return catOther;
 }
 
-}
-
-
 //
 // Token
 //
@@ -152,21 +114,64 @@ void debugToken(std::ostream & os, Token const & t, unsigned int flags)
 #endif
 
 
+//
+// Wrapper
+//
+
+void iparserdocstream::setEncoding(std::string const & e)
+{
+	is_ << lyx::setEncoding(e);
+}
+
+
+void iparserdocstream::putback(char_type c)
+{
+	s_ = c + s_;
+}
+
+
+void iparserdocstream::putback(docstring s)
+{
+	s_ = s + s_;
+}
+
+
+iparserdocstream & iparserdocstream::get(char_type &c)
+{
+	if (s_.empty())
+		is_.get(c);
+	else {
+		//cerr << "unparsed: " << to_utf8(s_) <<endl;
+		c = s_[0];
+		s_.erase(0,1);
+	}
+	return *this;
+}
+
+
 //
 // Parser
 //
 
 
-Parser::Parser(idocstream & is)
-	: lineno_(0), pos_(0), iss_(0), is_(is), encoding_latex_("utf8")
+Parser::Parser(idocstream & is, std::string const & fixedenc)
+	: lineno_(0), pos_(0), iss_(0), is_(is),
+	  encoding_iconv_(fixedenc.empty() ? "UTF-8" : fixedenc),
+	  theCatcodesType_(NORMAL_CATCODES), curr_cat_(UNDECIDED_CATCODES),
+	  fixed_enc_(!fixedenc.empty())
 {
+	if (fixed_enc_)
+		is_.setEncoding(fixedenc);
 }
 
 
 Parser::Parser(string const & s)
-	: lineno_(0), pos_(0), 
-	  iss_(new idocstringstream(from_utf8(s))), is_(*iss_), 
-	  encoding_latex_("utf8")
+	: lineno_(0), pos_(0),
+	  iss_(new idocstringstream(from_utf8(s))), is_(*iss_),
+	  encoding_iconv_("UTF-8"),
+	  theCatcodesType_(NORMAL_CATCODES), curr_cat_(UNDECIDED_CATCODES),
+	  // An idocstringstream can not change the encoding
+	  fixed_enc_(true)
 {
 }
 
@@ -177,16 +182,98 @@ Parser::~Parser()
 }
 
 
-void Parser::setEncoding(std::string const & e)
+void Parser::deparse()
 {
-	Encoding const * enc = encodings.fromLaTeXName(e);
+	string s;
+	for(size_type i = pos_ ; i < tokens_.size() ; ++i) {
+		s += tokens_[i].asInput();
+	}
+	is_.putback(from_utf8(s));
+	tokens_.erase(tokens_.begin() + pos_, tokens_.end());
+	// make sure that next token is read
+	tokenize_one();
+}
+
+
+bool Parser::setEncoding(std::string const & e, int const & p)
+{
+	// We may (and need to) use unsafe encodings here: Since the text is
+	// converted to unicode while reading from is_, we never see text in
+	// the original encoding of the parser, but operate on utf8 strings
+	// instead. Therefore, we cannot misparse high bytes as {, } or \\.
+	Encoding const * const enc = encodings.fromLaTeXName(e, p, true);
 	if (!enc) {
 		cerr << "Unknown encoding " << e << ". Ignoring." << std::endl;
-		return;
+		return false;
 	}
-	//cerr << "setting encoding to " << enc->iconvName() << std::endl;
-	is_ << lyx::setEncoding(enc->iconvName());
-	encoding_latex_ = e;
+	return setEncoding(enc->iconvName());
+}
+
+
+void Parser::catInit()
+{
+	if (curr_cat_ == theCatcodesType_)
+		return;
+	curr_cat_ = theCatcodesType_;
+
+	fill(theCatcode_, theCatcode_ + 256, catOther);
+	fill(theCatcode_ + 'a', theCatcode_ + 'z' + 1, catLetter);
+	fill(theCatcode_ + 'A', theCatcode_ + 'Z' + 1, catLetter);
+	// This is wrong!
+	theCatcode_[int('@')]  = catLetter;
+
+	if (theCatcodesType_ == NORMAL_CATCODES) {
+		theCatcode_[int('\\')] = catEscape;
+		theCatcode_[int('{')]  = catBegin;
+		theCatcode_[int('}')]  = catEnd;
+		theCatcode_[int('$')]  = catMath;
+		theCatcode_[int('&')]  = catAlign;
+		theCatcode_[int('\n')] = catNewline;
+		theCatcode_[int('#')]  = catParameter;
+		theCatcode_[int('^')]  = catSuper;
+		theCatcode_[int('_')]  = catSub;
+		theCatcode_[0x7f]      = catIgnore;
+		theCatcode_[int(' ')]  = catSpace;
+		theCatcode_[int('\t')] = catSpace;
+		theCatcode_[int('\r')] = catNewline;
+		theCatcode_[int('~')]  = catActive;
+		theCatcode_[int('%')]  = catComment;
+	}
+}
+
+CatCode Parser::catcode(char_type c) const
+{
+	if (c < 256)
+		return theCatcode_[(unsigned char)c];
+	return catOther;
+}
+
+
+void Parser::setCatcode(char c, CatCode cat)
+{
+	theCatcode_[(unsigned char)c] = cat;
+	deparse();
+}
+
+
+void Parser::setCatcodes(cat_type t)
+{
+	theCatcodesType_ = t;
+	deparse();
+}
+
+
+bool Parser::setEncoding(std::string const & e)
+{
+	//cerr << "setting encoding to " << e << std::endl;
+	encoding_iconv_ = e;
+	// If the encoding is fixed, we must not change the stream encoding
+	// (because the whole input uses that encoding, e.g. if it comes from
+	// the clipboard). We still need to track the original encoding in
+	// encoding_iconv_, so that the generated output is correct.
+	if (!fixed_enc_)
+		is_.setEncoding(e);
+	return true;
 }
 
 
@@ -216,7 +303,11 @@ Token const Parser::curr_token() const
 Token const Parser::next_token()
 {
 	static const Token dummy;
-	return good() ? tokens_[pos_] : dummy;
+	if (!good())
+		return dummy;
+	if (pos_ >= tokens_.size())
+		tokenize_one();
+	return pos_ < tokens_.size() ? tokens_[pos_] : dummy;
 }
 
 
@@ -224,12 +315,14 @@ Token const Parser::next_token()
 Token const Parser::next_next_token()
 {
 	static const Token dummy;
-	// If good() has not been called after the last get_token() we need
-	// to tokenize two more tokens.
-	if (pos_ + 1 >= tokens_.size()) {
+	if (!good())
+		return dummy;
+	// If tokenize_one() has not been called after the last get_token() we
+	// need to tokenize two more tokens.
+	if (pos_ >= tokens_.size())
 		tokenize_one();
+	if (pos_ + 1 >= tokens_.size())
 		tokenize_one();
-	}
 	return pos_ + 1 < tokens_.size() ? tokens_[pos_ + 1] : dummy;
 }
 
@@ -238,8 +331,16 @@ Token const Parser::next_next_token()
 Token const Parser::get_token()
 {
 	static const Token dummy;
-	//cerr << "looking at token " << tokens_[pos_] << " pos: " << pos_ << '\n';
-	return good() ? tokens_[pos_++] : dummy;
+	if (!good())
+		return dummy;
+	if (pos_ >= tokens_.size()) {
+		tokenize_one();
+		if (pos_ >= tokens_.size())
+			return dummy;
+	}
+	// cerr << "looking at token " << tokens_[pos_] 
+	//      << " pos: " << pos_ << '\n';
+	return tokens_[pos_++];
 }
 
 
@@ -331,6 +432,13 @@ void Parser::popPosition()
 {
 	pos_ = positions_.back();
 	positions_.pop_back();
+	deparse();
+}
+
+
+void Parser::dropPosition()
+{
+	positions_.pop_back();
 }
 
 
@@ -338,16 +446,9 @@ bool Parser::good()
 {
 	if (pos_ < tokens_.size())
 		return true;
-	tokenize_one();
-	return pos_ < tokens_.size();
-}
-
-
-char Parser::getChar()
-{
-	if (!good())
-		error("The input stream is not well...");
-	return get_token().character();
+	if (!is_.good())
+		return false;
+	return is_.peek() != idocstream::traits_type::eof();
 }
 
 
@@ -463,7 +564,7 @@ string Parser::getFullParentheseArg()
 }
 
 
-string const Parser::verbatimEnvironment(string const & name)
+string const Parser::ertEnvironment(string const & name)
 {
 	if (!good())
 		return string();
@@ -476,7 +577,7 @@ string const Parser::verbatimEnvironment(string const & name)
 		} else if (t.asInput() == "\\begin") {
 			string const env = getArg('{', '}');
 			os << "\\begin{" << env << '}'
-			   << verbatimEnvironment(env)
+			   << ertEnvironment(env)
 			   << "\\end{" << env << '}';
 		} else if (t.asInput() == "\\end") {
 			string const end = getArg('{', '}');
@@ -494,10 +595,10 @@ string const Parser::verbatimEnvironment(string const & name)
 
 
 string const Parser::plainEnvironment(string const & name)
-{ 
+{
 	if (!good())
 		return string();
-	
+
 	ostringstream os;
 	for (Token t = get_token(); good(); t = get_token()) {
 		if (t.asInput() == "\\end") {
@@ -518,7 +619,7 @@ string const Parser::plainCommand(char left, char right, string const & name)
 {
 	if (!good())
 		return string();
-	// ceck if first token is really the start character
+	// check if first token is really the start character
 	Token tok = get_token();
 	if (tok.character() != left) {
 		cerr << "first character does not match start character of command \\" << name << endl;
@@ -533,103 +634,63 @@ string const Parser::plainCommand(char left, char right, string const & name)
 	}
 	cerr << "unexpected end of input" << endl;
 	return os.str();
-} 
+}
 
 
-void Parser::tokenize_one()
+Parser::Arg Parser::verbatimStuff(string const & end_string, bool const allow_linebreak)
 {
-	catInit();
-	char_type c;
-	if (!is_.get(c)) 
-		return;
+	if (!good())
+		return Arg(false, string());
 
-	switch (catcode(c)) {
-	case catSpace: {
-		docstring s(1, c);
-		while (is_.get(c) && catcode(c) == catSpace)
-			s += c;
-		if (catcode(c) != catSpace)
-			is_.putback(c);
-		push_back(Token(s, catSpace));
-		break;
-	}
-		
-	case catNewline: {
-		++lineno_;
-		docstring s(1, getNewline(is_, c));
-		while (is_.get(c) && catcode(c) == catNewline) {
-			++lineno_;
-			s += getNewline(is_, c);
-		}
-		if (catcode(c) != catNewline)
-			is_.putback(c);
-		push_back(Token(s, catNewline));
-		break;
-	}
-		
-	case catComment: {
-		// We don't treat "%\n" combinations here specially because
-		// we want to preserve them in the preamble
-		docstring s;
-		while (is_.get(c) && catcode(c) != catNewline)
-			s += c;
-		// handle possible DOS line ending
-		if (catcode(c) == catNewline)
-			c = getNewline(is_, c);
-		// Note: The '%' at the beginning and the '\n' at the end
-		// of the comment are not stored.
-		++lineno_;
-		push_back(Token(s, catComment));
-		break;
-	}
-		
-	case catEscape: {
-		is_.get(c);
-		if (!is_) {
-			error("unexpected end of input");
+	pushPosition();
+	ostringstream oss;
+	size_t match_index = 0;
+	setCatcodes(VERBATIM_CATCODES);
+	for (Token t = get_token(); good(); t = get_token()) {
+		// FIXME t.asInput() might be longer than we need ?
+		if (t.asInput() == end_string.substr(match_index,
+						     t.asInput().length())) {
+			match_index += t.asInput().length();
+			if (match_index >= end_string.length())
+				break;
 		} else {
-			docstring s(1, c);
-			if (catcode(c) == catLetter) {
-				// collect letters
-				while (is_.get(c) && catcode(c) == catLetter)
-					s += c;
-				if (catcode(c) != catLetter)
-					is_.putback(c);
+			if (!allow_linebreak && t.asInput() == "\n") {
+				cerr << "unexpected end of input" << endl;
+				popPosition();
+				setCatcodes(NORMAL_CATCODES);
+				return Arg(false, string());
 			}
-			push_back(Token(s, catEscape));
+			if (match_index) {
+				oss << end_string.substr(0, match_index) 
+				    << t.asInput();
+				match_index = 0;
+			} else
+				oss << t.asInput();
 		}
-		break;
 	}
-		
-	case catIgnore: {
-		cerr << "ignoring a char: " << c << "\n";
-		break;
+
+	if (!good()) {
+		cerr << "unexpected end of input" << endl;
+		popPosition();
+		setCatcodes(NORMAL_CATCODES);
+		return Arg(false, string());
 	}
-		
-	default:
-		push_back(Token(docstring(1, c), catcode(c)));
-	}
-	//cerr << tokens_.back();
+	setCatcodes(NORMAL_CATCODES);
+	dropPosition();
+	return Arg(true, oss.str());
 }
 
 
-void Parser::dump() const
+string const Parser::verbatimEnvironment(string const & name)
 {
-	cerr << "\nTokens: ";
-	for (unsigned i = 0; i < tokens_.size(); ++i) {
-		if (i == pos_)
-			cerr << " <#> ";
-		cerr << tokens_[i];
-	}
-	cerr << " pos: " << pos_ << "\n";
-}
-
-
-void Parser::error(string const & msg)
-{
-	cerr << "Line ~" << lineno_ << ":  parse error: " << msg << endl;
-	dump();
-	//exit(1);
+	//FIXME: do something if endstring is not found
+	string s = verbatimStuff("\\end{" + name + "}").second;
+	// ignore one newline at beginning or end of string
+	if (prefixIs(s, "\n"))
+		s.erase(0,1);
+	if (suffixIs(s, "\n"))
+		s.erase(s.length() - 1,1);
+	return s;
 }
 
 
@@ -672,22 +733,107 @@ string Parser::verbatim_item()
 }
 
 
+void Parser::tokenize_one()
+{
+	catInit();
+	char_type c;
+	if (!is_.get(c))
+		return;
+
+	switch (catcode(c)) {
+	case catSpace: {
+		docstring s(1, c);
+		while (is_.get(c) && catcode(c) == catSpace)
+			s += c;
+		if (catcode(c) != catSpace)
+			is_.putback(c);
+		push_back(Token(s, catSpace));
+		break;
+	}
+
+	case catNewline: {
+		++lineno_;
+		docstring s(1, getNewline(is_, c));
+		while (is_.get(c) && catcode(c) == catNewline) {
+			++lineno_;
+			s += getNewline(is_, c);
+		}
+		if (catcode(c) != catNewline)
+			is_.putback(c);
+		push_back(Token(s, catNewline));
+		break;
+	}
+
+	case catComment: {
+		// We don't treat "%\n" combinations here specially because
+		// we want to preserve them in the preamble
+		docstring s;
+		while (is_.get(c) && catcode(c) != catNewline)
+			s += c;
+		// handle possible DOS line ending
+		if (catcode(c) == catNewline)
+			c = getNewline(is_, c);
+		// Note: The '%' at the beginning and the '\n' at the end
+		// of the comment are not stored.
+		++lineno_;
+		push_back(Token(s, catComment));
+		break;
+	}
+
+	case catEscape: {
+		is_.get(c);
+		if (!is_) {
+			error("unexpected end of input");
+		} else {
+			docstring s(1, c);
+			if (catcode(c) == catLetter) {
+				// collect letters
+				while (is_.get(c) && catcode(c) == catLetter)
+					s += c;
+				if (catcode(c) != catLetter)
+					is_.putback(c);
+			}
+			push_back(Token(s, catEscape));
+		}
+		break;
+	}
+
+	case catIgnore: {
+		cerr << "ignoring a char: " << c << "\n";
+		break;
+	}
+
+	default:
+		push_back(Token(docstring(1, c), catcode(c)));
+	}
+	//cerr << tokens_.back();
+}
+
+
+void Parser::dump() const
+{
+	cerr << "\nTokens: ";
+	for (unsigned i = 0; i < tokens_.size(); ++i) {
+		if (i == pos_)
+			cerr << " <#> ";
+		cerr << tokens_[i];
+	}
+	cerr << " pos: " << pos_ << "\n";
+}
+
+
+void Parser::error(string const & msg)
+{
+	cerr << "Line ~" << lineno_ << ":  parse error: " << msg << endl;
+	dump();
+	//exit(1);
+}
+
+
 void Parser::reset()
 {
 	pos_ = 0;
 }
 
 
-void Parser::setCatCode(char c, CatCode cat)
-{
-	theCatcode[(unsigned char)c] = cat;
-}
-
-
-CatCode Parser::getCatCode(char c) const
-{
-	return theCatcode[(unsigned char)c];
-}
-
-
 } // namespace lyx
