@@ -39,7 +39,17 @@ public:
 	///
 	std::string notefontcolor() const { return h_notefontcolor; }
 	///
-	std::string language() const { return h_language; }
+	bool refstyle() const { return h_use_refstyle; }
+	///
+	bool fontCJKSet() const { return h_font_cjk_set; }
+	///
+	std::string fontCJK() const { return h_font_cjk; }
+	///
+	void fontCJK(std::string const & f) { h_font_cjk_set = true; h_font_cjk = f; }
+	/// The document language
+	std::string docLanguage() const { return h_language; }
+	/// The language of text which is not explicitly marked
+	std::string defaultLanguage() const  { return default_language; }
 	///
 	std::string use_indices() const { return h_use_indices; }
 	///
@@ -60,6 +70,10 @@ public:
 	///
 	void suppressDate(bool suppress);
 	///
+	std::string citeEngine() const { return h_cite_engine; }
+	///
+	void citeEngine(std::string const & e) { h_cite_engine = e; }
+	///
 	bool titleLayoutFound() const { return title_layout_found; }
 	///
 	void titleLayoutFound(bool found) { title_layout_found = found; }
@@ -89,18 +103,27 @@ private:
 
 	/// needed to handle encodings with babel
 	bool one_language;
+	/// the main non-CJK language
+	std::string default_language;
+	/// Was babel called explicitly?
+	bool explicit_babel;
 
 	/// was at least one title layout found?
 	bool title_layout_found;
 
 	std::ostringstream h_preamble;
 	std::string h_backgroundcolor;
+	std::string h_biblio_style;
+	std::string h_bibtex_command;
 	std::string h_boxbgcolor;
 	std::string h_cite_engine;
+	std::string h_cite_engine_type;
+	std::string h_color;
 	std::string h_defskip;
 	std::string h_float_placement;
 	std::string h_fontcolor;
 	std::string h_fontencoding;
+	std::string h_font_math;
 	std::string h_font_roman;
 	std::string h_font_sans;
 	std::string h_font_typewriter;
@@ -110,15 +133,21 @@ private:
 	std::string h_font_osf;
 	std::string h_font_sf_scale;
 	std::string h_font_tt_scale;
+	bool h_font_cjk_set;
+	std::string h_font_cjk;
 	std::string h_graphics;
 	std::string h_default_output_format;
 	std::string h_html_be_strict;
 	std::string h_html_css_as_file;
 	std::string h_html_math_output;
+	std::string h_index[99];
+	std::string h_index_command;
 	std::string h_inputencoding;
+	std::string h_justification;
 	std::string h_language;
 	std::string h_language_package;
 	std::string h_listings_params;
+	std::string h_maintain_unincluded_children;
 	std::string h_margins;
 	std::string h_notefontcolor;
 	std::string h_options;
@@ -151,6 +180,7 @@ private:
 	std::string h_pdf_quoted_options;
 	std::string h_quotes_language;
 	std::string h_secnumdepth;
+	std::string h_shortcut[99];
 	std::string h_spacing;
 	std::string h_suppress_date;
 	std::string h_textclass;
@@ -159,13 +189,10 @@ private:
 	std::string h_use_bibtopic;
 	std::string h_use_indices;
 	std::string h_use_geometry;
-	std::string h_use_amsmath;
+	std::map<std::string, std::string> h_use_packages;
 	std::string h_use_default_options;
-	std::string h_use_esint;
 	std::string h_use_hyperref;
-	std::string h_use_mhchem;
-	std::string h_use_mathdots;
-	std::string h_use_refstyle;
+	bool h_use_refstyle;
 
 	/*!
 	 * Add package \p name with options \p options to used_packages.
